
/////////////////////////////////////////
// START of File: bz_zcht.cpp
/////////////////////////////////////////

//-------------------------------------------------------------------
// COPYRIGHT NOTICE:  This code is protected by Copyright!
//
// Copyright (c) 1994 through 1997 by DPC Technology Corporation.
// All rights reserved.
//
// See files "cf_copyr.h" and "cf_licen.h" for details of
// copyright and license specifications, and contact information.
//-------------------------------------------------------------------

#include	<cf_api/bf_defs.h>
#include <cf_api/bz_zcht.hpp>

ZafClassID ZafChart::classID = ID_ZAF_CHART;
ZafClassNameChar ZAF_FARDATA ZafChart::className[] = ZAF_ITEXT ("ZafChart");
           




//-------------------------------------------------------------------
//
//-------------------------------------------------------------------

ZafChart::ZafChart (int left, int top, int width, int height,
					int iChartTypeID, int iIndexMethodID) :
	ZafChartStub (left, top, width, height, 
					iChartTypeID, iIndexMethodID)
{
	SetNoncurrent (true);
	SetOSDraw (false);

	chartView = new cf_ChartView (
			ConvertChartType (iChartTypeID), 
			CF_DISP_2D, 
			ConvertIndexMode (iIndexMethodID));
	marksXAxis = true;
	marksYAxis = true;
}





//-------------------------------------------------------------------
//
//-------------------------------------------------------------------

ZafChart::ZafChart (const ZafChart &copy) :
	ZafChartStub (copy)
{
	SetNoncurrent (true);
	SetOSDraw (false);
	chartView = new cf_ChartView (
			ConvertChartType (chartType), 
			CF_DISP_2D, 
			ConvertIndexMode (indexMethod));
	SetAxisColor (axisColor);
	SetMarksXAxis (marksXAxis);
	SetMarksYAxis (marksYAxis);
	SetTextTitle (textTitle);
	SetTextXAxis (textXAxis);
	SetTextYAxis (textYAxis);
}





//-------------------------------------------------------------------
//
//-------------------------------------------------------------------

ZafChart::~ZafChart ()
{
	if (chartView != NULL)
		delete chartView;
}





//-------------------------------------------------------------------
//
//-------------------------------------------------------------------

cf_ChartModel*
ZafChart::ActiveModel ()
{
	return (chartView->activeModel ());
}





//-------------------------------------------------------------------
//
//-------------------------------------------------------------------

ZafError
ZafChart::AddDataGroup (int groupIndex)
{
	chartView->activeModel ()->activateGroupUserData (groupIndex);
	return (ZAF_ERROR_NONE);
}





//-------------------------------------------------------------------
//
//-------------------------------------------------------------------

ZafError 
ZafChart::AddDataPoint (int groupIndex, double dataValue)
{
	chartView->addData (groupIndex, dataValue);
	return (ZAF_ERROR_NONE);
}





//-------------------------------------------------------------------
//
//-------------------------------------------------------------------

ZafLogicalColor 
ZafChart::AxisColor (ZafLogicalColor *mono)
{
	return (ZafChartStub::AxisColor (mono));
}





//-------------------------------------------------------------------
//
//-------------------------------------------------------------------

ZafLogicalColor 
ZafChart::SetAxisColor (ZafLogicalColor color, ZafLogicalColor mono)
{
	chartView->attribAxisItemDisp (CF_AXIS_X_BOTTOM)->colorPen (color);
	chartView->attribAxisItemDisp (CF_AXIS_Y_LEFT)->colorPen (color);
	Redraw ();

	return (ZafChartStub::SetAxisColor (color, mono));
}





//-------------------------------------------------------------------
//
//-------------------------------------------------------------------

int 
ZafChart::ChartType () const
{
	return (ZafChartStub::ChartType ());
}





//-------------------------------------------------------------------
//
//-------------------------------------------------------------------

int 
ZafChart::SetChartType (int chartTypeID)
{
	chartView->chartType (ConvertChartType (chartTypeID));
	Redraw ();

	return (ZafChartStub::SetChartType (chartTypeID));
}





//-------------------------------------------------------------------
//
//-------------------------------------------------------------------

ZafError 
ZafChart::ClearDataAll ()
{
	chartView->activeModel ()-> dataGroupsReset ();
	return (ZAF_ERROR_NONE);
}





//-------------------------------------------------------------------
//
//-------------------------------------------------------------------

ZafError 
ZafChart::ClearDataGroup (int groupIndex)
{
	chartView->activeModel ()
			-> activateGroupUserData (groupIndex)
					->reset ();
	return (ZAF_ERROR_NONE);
}





//-------------------------------------------------------------------
//
//-------------------------------------------------------------------

cf_DataGroup*
ZafChart::DataGroup (int groupIndex)
{
	return (chartView->activeModel ()
			-> activateGroupUserData (groupIndex));
}





//-------------------------------------------------------------------
//
//-------------------------------------------------------------------

ZafLogicalColor 
ZafChart::DataGroupBackgroundColor (int group, ZafLogicalColor* /* mono */)
{
	return chartView->attribDataGroupDisp (group)->colorBrush ();
}





//-------------------------------------------------------------------
//
//-------------------------------------------------------------------

ZafLogicalColor 
ZafChart::SetDataGroupBackgroundColor (	int group, 
										ZafLogicalColor color, 
										ZafLogicalColor )
{
	chartView -> attribDataGroupDisp (group) -> colorBrush (color);
	if (AutomaticUpdate ())
		Redisplay ();
	return (color);
}





//-------------------------------------------------------------------
//
//-------------------------------------------------------------------

ZafLogicalFillPattern 
ZafChart::DataGroupFillPattern (int groupIndex)
{
	return ((ZafLogicalFillPattern) chartView->attribDataGroupDisp 
			(groupIndex)->styleBrushHatch ());
}





//-------------------------------------------------------------------
//
//-------------------------------------------------------------------

ZafLogicalFillPattern 
ZafChart::SetDataGroupFillPattern (	int groupIndex, 
									ZafLogicalFillPattern fillPattern)
{
	chartView -> attribDataGroupDisp (groupIndex) -> styleBrushHatch (
			(bf_BrushHatch) fillPattern);
	if (AutomaticUpdate ())
		Redisplay ();
	return (fillPattern);
}





//-------------------------------------------------------------------
//
//-------------------------------------------------------------------

ZafLogicalFont 
ZafChart::DataGroupFont (int /* groupIndex */)
{
	// ###
	return (ZAF_FNT_DEFAULT);
}





//-------------------------------------------------------------------
//
//-------------------------------------------------------------------

ZafLogicalFont 
ZafChart::SetDataGroupFont (int /* groupIndex */, ZafLogicalFont font)
{
	// ###
	return (font);
}





//-------------------------------------------------------------------
//
//-------------------------------------------------------------------

ZafLogicalColor 
ZafChart::DataGroupForegroundColor (int , ZafLogicalColor* )
{
	return (ZAF_CLR_BLACK);
}





//-------------------------------------------------------------------
//
//-------------------------------------------------------------------

ZafLogicalColor 
ZafChart::SetDataGroupForegroundColor (	int groupIndex, 
										ZafLogicalColor color, 
										ZafLogicalColor )
{
	chartView -> attribDataGroupDisp (groupIndex) -> colorPen (
			color);
	if (AutomaticUpdate ())
		Redisplay ();
	return (color);
}





//-------------------------------------------------------------------
//
//-------------------------------------------------------------------

ZafLogicalLineStyle 
ZafChart::DataGroupLineStyle (int groupIndex)
{
	return ((ZafLogicalLineStyle) chartView->attribDataGroupDisp 
			(groupIndex) -> stylePen ());
}





//-------------------------------------------------------------------
//
//-------------------------------------------------------------------

ZafLogicalLineStyle 
ZafChart::SetDataGroupLineStyle (	int groupIndex, 
									ZafLogicalLineStyle lineStyle)
{
	chartView -> attribDataGroupDisp (groupIndex) -> stylePen (
			(bf_PenStyle) lineStyle);
	if (AutomaticUpdate ())
		Redisplay ();
	return (lineStyle);
}





//-------------------------------------------------------------------
//
//-------------------------------------------------------------------

cf_XYZ*
ZafChart::DataPoint (int /* groupIndex */, int /* dataIndex */)
{
	// ###	
	return (NULL);
}





//-------------------------------------------------------------------
//
//-------------------------------------------------------------------

int 
ZafChart::IndexMethod () const
{
	return (ZafChartStub::IndexMethod ());
}





//-------------------------------------------------------------------
//
//-------------------------------------------------------------------

int 
ZafChart::SetIndexMethod (int indexMethodID)
{
	chartView->chartIndexMode (
			ConvertIndexMode (indexMethodID));

	Redraw ();
	return (ZafChartStub::SetIndexMethod (indexMethodID));
}





//-------------------------------------------------------------------
//
//-------------------------------------------------------------------

bool 
ZafChart::MarksXAxis () const
{
	return (marksXAxis);
}





//-------------------------------------------------------------------
//
//-------------------------------------------------------------------

bool 
ZafChart::SetMarksXAxis (bool xAxisMarks)
{
	if (xAxisMarks)
		chartView->activeAxis (CF_AXIS_X_BOTTOM)->on ();
	else
		chartView->activeAxis (CF_AXIS_X_BOTTOM)->off ();
	if (AutomaticUpdate ())
		Redisplay ();

	return (marksXAxis = xAxisMarks);
}





//-------------------------------------------------------------------
//
//-------------------------------------------------------------------

bool 
ZafChart::MarksYAxis () const
{
	return (marksYAxis);
}





//-------------------------------------------------------------------
//
//-------------------------------------------------------------------

bool 
ZafChart::SetMarksYAxis (bool yAxisMarks)
{
	if (yAxisMarks)
		chartView->activeAxis (CF_AXIS_Y_LEFT)->on ();
	else
		chartView->activeAxis (CF_AXIS_Y_LEFT)->off ();
	if (AutomaticUpdate ())
		Redisplay ();

	return (marksYAxis = yAxisMarks);
}





//-------------------------------------------------------------------
//
//-------------------------------------------------------------------

const ZafIChar*
ZafChart::TextTitle () 
{
	return (ZafChartStub::TextTitle ());
}





//-------------------------------------------------------------------
//
//-------------------------------------------------------------------

const ZafIChar*
ZafChart::SetTextTitle (const ZafIChar *text)
{
	chartView->setLabelTitle (text, CF_TITLE_MAIN);
	Redraw ();

	return (ZafChartStub::SetTextTitle (text));
}





//-------------------------------------------------------------------
//
//-------------------------------------------------------------------

const ZafIChar*
ZafChart::TextXAxis ()
{
	return (ZafChartStub::TextXAxis ());
}





//-------------------------------------------------------------------
//
//-------------------------------------------------------------------

const ZafIChar *
ZafChart::SetTextXAxis (const ZafIChar *text)
{
	chartView->setLabelAxisName (text, CF_AXIS_X_BOTTOM,
			-100, -125, 125);
	Redraw ();

	return (ZafChartStub::SetTextXAxis (text));
}





//-------------------------------------------------------------------
//
//-------------------------------------------------------------------

const ZafIChar*
ZafChart::TextYAxis ()
{
	return (ZafChartStub::TextYAxis ());
}





//-------------------------------------------------------------------
//
//-------------------------------------------------------------------

const ZafIChar *ZafChart::SetTextYAxis (const ZafIChar *text)
{
	chartView->setLabelAxisName (text, CF_AXIS_Y_LEFT);
	Redraw ();

	return (ZafChartStub::SetTextYAxis (text));
}





//-------------------------------------------------------------------
//
//-------------------------------------------------------------------

#if defined(ZAF_DEFAULT_DRAW)
ZafEventType 
ZafChart::Draw (const ZafEventStruct& event, ZafEventType ccode)
{
	ZafRegionStruct drawRegion = BeginDraw ();

#if defined(ZAF_MSDOS)
	this->display->SetClipRegion(event.region);
#endif

	if (bordered)
    	DrawBorder (drawRegion, ccode);
	DrawBackground (drawRegion, ccode);

	bf_Display		disp (this -> display, this);

	if (chartView->activatedIs () == FALSE ) 
			chartView->activate (&disp);
	chartView->draw (&disp);

	EndDraw ();

	return (event.type);
}
#endif





//-------------------------------------------------------------------
//
//-------------------------------------------------------------------

ZafWindowObject*
ZafChart::Duplicate (void)
{
	return (new ZafChart (*this));
}





//-------------------------------------------------------------------
//
//-------------------------------------------------------------------

ZafEventType 
ZafChart::Event (const ZafEventStruct& event)
{
	ZafEventType ccode = event.type;

	switch (ccode)
	{
	case N_SIZE:
		// Let the chart-view/chart-model process the re-size.
		{
			bf_Display disp (this->display, this);
			chartView->updateDisplayMetrics (&disp);
			Redisplay ();
		}
		break;

	default:
		ccode = ZafChartStub::Event (event);
		break;
	}
	return (ccode);
}

// --- Persistence ---





//-------------------------------------------------------------------
//
//-------------------------------------------------------------------

ZafChart::ZafChart (const ZafIChar *name, ZafObjectPersistence &persist) :
	ZafChartStub (name, persist.PushLevel (className, classID, ZAF_PERSIST_DIRECTORY))
{
	persist.PopLevel ();

	chartView = new cf_ChartView (
			ConvertChartType (chartType), 
			CF_DISP_2D, 
			ConvertIndexMode (indexMethod));
	SetAxisColor (axisColor);
	SetMarksXAxis (marksXAxis);
	SetMarksYAxis (marksYAxis);
	SetTextTitle (textTitle);
	SetTextXAxis (textXAxis);
	SetTextYAxis (textYAxis);
}





//-------------------------------------------------------------------
//
//-------------------------------------------------------------------

ZafElement *ZafChart::Read (const ZafIChar *name, ZafObjectPersistence &persist)
{
	return (new ZafChart (name, persist));
}





//-------------------------------------------------------------------
//
//-------------------------------------------------------------------

void ZafChart::Write (ZafObjectPersistence &persist)
{
	ZafChartStub::Write (persist.PushLevel (className, classID, ZAF_PERSIST_DIRECTORY));
	persist.PopLevel ();
}




//-------------------------------------------------------------------
//
//-------------------------------------------------------------------

cf_ChartType
ZafChart::ConvertChartType ( int zaf_chart_id)
{
	cf_ChartType		cf_chart_id;


	switch (zaf_chart_id)
	{	
		case ZAF_PIE_2D:	cf_chart_id = CF_TYPE_PIE; break;
		case ZAF_BAR_2D:	cf_chart_id = CF_TYPE_BAR; break;
		case ZAF_COL_2D:	cf_chart_id = CF_TYPE_COLUMN; break;
		case ZAF_STACK_2D:	cf_chart_id = CF_TYPE_STACK; break;
		case ZAF_POINTS_2D:	cf_chart_id = CF_TYPE_PSER; break;
		case ZAF_LINE_2D:	cf_chart_id = CF_TYPE_LINE; break;
		case ZAF_AREA_2D:	cf_chart_id = CF_TYPE_AREA; break;
		case ZAF_RIBBON_2D:	cf_chart_id = CF_TYPE_RIBBON; break;
		default:			cf_chart_id = CF_TYPE_PSER; break;
	}

	return (cf_chart_id);
}





//-------------------------------------------------------------
// 
//-------------------------------------------------------------

cf_IndexMode
ZafChart::ConvertIndexMode ( int zaf_index_mode)
{
	cf_IndexMode		cf_index_mode;

	switch (zaf_index_mode)
	{
		case ZAF_RANGE_NATURAL:	cf_index_mode = CF_INDEX_NAT_N; break;
		case ZAF_RANGE_INTEGER:	cf_index_mode = CF_INDEX_INT_I; break;
		default:				cf_index_mode = CF_INDEX_NAT_N; break;
	}

	return (cf_index_mode);
}





//-------------------------------------------------------------
// 
//-------------------------------------------------------------

void
ZafChart::Redraw ()
{
	bf_Display		disp (this -> display, this);


	if (chartView->activatedIs () == FALSE ) 
			chartView->activate (&disp);
	if (AutomaticUpdate ())
		Redisplay ();
}





//-------------------------------------------------------------
// 
//-------------------------------------------------------------

/////////////////////////////////////////
// END of File: bz_zcht.cpp
/////////////////////////////////////////

